#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>

#include <pthread.h>
#include <termios.h>
#include <sys/types.h>
        #include <sys/stat.h>
        #include <fcntl.h>



#define BUFLEN 512
#define NPACK 10
#define PORT 9930
#define SRV_IP "127.0.01"
//#define SRV_IP "192.168.1.40"


#define THREAD_DELAY 100000


#define BAUDRATE B38400
#define MODEMDEVICE "/dev/ttyS0"



enum
{
  up = 0,
  down,
  left,
  right,
  A,
  B,
  X,
  Y,
  start,
  back,
  totalButtons,
};

int buttonStatus[totalButtons];

#define DEBUG 0


/*************************************************************/
void diep(char *s)
{
  perror(s);
  exit(1);
}

/*************************************************************/

struct sockaddr_in si_me, si_other;
int playerSocket; 

void InitServerSocket();
void ProcessMsg(char * buf);
void RecvCmd();
void * SerialWriteThread(void *param);
void InitButtons();
void BuildSerialMessage(char * _buf);

/*************************************************************/
int main(int argc, char ** argv)
{
  int sw;
  pthread_t serialWriteId;

  InitButtons();

  sw = pthread_create(&serialWriteId, NULL, SerialWriteThread, NULL);
  if (sw != 0)
  {
    printf("Error creating serial thread, exiting...\n");
    exit(0);
  }
  InitServerSocket();
  while (1)
  {
//    printf("In mail while loop!\n");
//    usleep(THREAD_DELAY);
    RecvCmd();
  }
  return 0;
}
/*************************************************************/
void InitButtons()
{
  int x = 0;
  for (x = 0; x< totalButtons; x++)
  {
    buttonStatus[x] = 0;
  }
}
/*************************************************************/
void RecvCmd()
{
  int slen=sizeof(si_other);
  char buf[BUFLEN];

  if (recvfrom(playerSocket, buf, BUFLEN, 0, &si_other, &slen)==-1)
    diep("recvfrom()");
  ProcessMsg(buf);
}
/*************************************************************/
void ProcessMsg(char * buf)
{
  int x = 0;
//  printf("Received msg: %s\n", buf);
  if (buf[0] == '#' && buf[1] == '#')
  {
    x = 2;
    while (buf[x] != '!' && x < BUFLEN)
    {
      if (buf[x] == 'U')
      {
        buttonStatus[up] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'D')
      {
        buttonStatus[down] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'L')
      {
        buttonStatus[left] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'R')
      {
        buttonStatus[right] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'A')
      {
        buttonStatus[A] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'B')
      {
        buttonStatus[B] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'X')
      {
        buttonStatus[X] = atoi(&buf[x+1]);
        x+=2;
      }
      else if (buf[x] == 'Y')
      {
        buttonStatus[Y] = atoi(&buf[x+1]);
        x+=2;
      }
    }
  }
}

/*************************************************************/
void InitServerSocket()
{
  int x, slen=sizeof(si_other);
  char buf[BUFLEN];

  if ((playerSocket=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1)
    diep("socket");

  memset((char *) &si_me, sizeof(si_me), 0);
  si_me.sin_family = AF_INET;
  si_me.sin_port = htons(PORT);
  si_me.sin_addr.s_addr = htonl(INADDR_ANY);
  if (bind(playerSocket, &si_me, sizeof(si_me))==-1)
    diep("bind");
  printf("SOCKET UP\n");
  if (recvfrom(playerSocket, buf, BUFLEN, 0, &si_other, &slen)==-1)
    diep("recvfrom()");
  printf("Client COnnected!\n");
  printf("Port: %d\n", si_other.sin_port);
  char * test = (char *)&si_other;
  for (x = 0; x< sizeof(si_other); x++)
    printf("%x ", test[x]);
}
/*************************************************************/
void * SerialWriteThread(void * param)
{
  char buf[BUFLEN];
  char msg[BUFLEN];
  int fd,c, res, x;
  struct termios oldtio,newtio;
  fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY );
  if (fd <0) {perror(MODEMDEVICE); exit(-1); }

  tcgetattr(fd,&oldtio); /* save current serial port settings */
  bzero(&newtio, sizeof(newtio)); /* clear struct for new port settings */

  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD ;

  //newtio.c_iflag = IGNPAR | ICRNL;

  newtio.c_oflag = 0;

//  newtio.c_lflag = ICANON;

  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

#if 0

speed 38400 baud;
-brkint ixoff -imaxbel



#endif
  printf("Serial Write Thread Starting Up!\n");
  while(1)
  {
//    sprintf(buf, "AA");
//    write(fd,buf,1);
    BuildSerialMessage(buf);
    printf("Send Serial Message: %s\n", buf);
    for (x = 0; x< 10; x++)
    {
      write(fd,&buf[x],1);
      usleep(1000);
    }
//    write(fd,buf,10);
    memset(buf,0,BUFLEN);
    usleep(THREAD_DELAY);

  }
}
/*************************************************************/
void BuildSerialMessage(char * _buf)
{
//  sprintf(_buf,"##U%dD%dL%dR%dA%dB%dX%dY%d!!", 
  sprintf(_buf,"A%d%d%d%d%d%d%d%d", 
                       buttonStatus[0],
//                       1,
                       buttonStatus[1],
                       buttonStatus[2],
                       buttonStatus[3],
                       buttonStatus[4],
                       buttonStatus[5],
                       buttonStatus[6],
                       buttonStatus[7]);


}
/*************************************************************/
